<?php
/*
Plugin Name: Semisecure Login Reimagined
Plugin URI: http://moggy.laceous.com/2008/05/30/semisecure-login-reimagined/
Description: A version of Semisecure Login for WP2.x that uses RSA public-key encryption. This makes it compatible with the new phpass hashed passwords, as well as with the older MD5 method. You can generate a new keypair or uninstall this plugin by visiting the <a href="options-general.php?page=semisecureloginreimagined">settings</a> page.
Version: 1.4.0.2
Author: moggy
Author URI: http://moggy.laceous.com/
*/

/*
    Copyright 2008-2009

    Based on Semisecure Login (http://wordpress.org/extend/plugins/semisecure-login/)
    and Semisecure Login for WordPress 2.5 (http://wordpress.org/extend/plugins/semisecure-login-for-25/)

    Thanks to BigIntegers and RSA in JavaScript (http://www-cs-students.stanford.edu/~tjw/jsbn/)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

require_once('SemisecureLoginReimagined_AdvancedOptions.php');
if (!class_exists('SemisecureLoginReimagined')) {
	class SemisecureLoginReimagined extends SemisecureLoginReimagined_AdvancedOptions {

		// grrr... PHP4-style constructor... conforming to WordPress minimum PHP requirements (PHP4 is dead!)
		function SemisecureLoginReimagined() {
			// Plugin Hooks
			// checking that the plugin hook functions exist so that when accessing the file directly it doesn't throw a fatal error (and possibly give away info about your server)
			if (function_exists('register_activation_hook'))
				register_activation_hook(__FILE__, array(&$this, 'semisecure_activate'));
			if (function_exists('add_action')) {
				add_action('login_head', array(&$this, 'login_head'));
				add_action('login_form', array(&$this, 'login_form'), $this->loginFormSemisecureMessagePosition);
				add_action('wp_authenticate', array(&$this, 'semisecure_authenticate'), 1, 2);
				add_action('admin_menu', array(&$this, 'add_pages'));
				add_action('init', array(&$this, 'wp_init'));
			}
		}

		function initializePluginTextdomain() {
			// pre-WP2.6 compatibility
			if (! defined('PLUGINDIR')) {
				define( 'PLUGINDIR', 'wp-content/plugins' );  // Relative to ABSPATH.  For back compat.
			}

			load_plugin_textdomain('semisecure-login-reimagined', PLUGINDIR . '/' . dirname(plugin_basename(__FILE__)) . '/languages');
		}

		// Plugin Hook
		// call session_start before sending any headers
		// this fixes an issue where sometimes you would have to reload the page to properly start the session
		// this will only be called if an advanced option has been set
		function wp_init() {
			// load the WP textdomain for this plugin (for translations)
			$this->initializePluginTextdomain();

			if ($this->forceSessionStartBeforeSendingHeaders) {
				if ($this->forceSessionStartOnlyForLoginPage) {
					// only start the session if this is the login page
					if (substr($_SERVER['SCRIPT_FILENAME'], -12) == 'wp-login.php' || substr($_SERVER['SCRIPT_NAME'], -12) == 'wp-login.php' || substr($_SERVER['PHP_SELF'], -12) == 'wp-login.php') {
						@session_start();
					}
				}
				else {
					// start the session for every page
					@session_start();
				}
			}
		}

		// Plugin Hook
		// generates and stores your unique rsa keys when the plugin is first activated
		function semisecure_activate() {
			// check if the option already exists in the database
			if (get_option('semisecurelogin_reimagined_rsa_keys') === FALSE) {
				// create an initial keypair
				$successful = $this->storeNewRsaKeypair(1024, 10001);

				// If I deactivate the plugin here then the user is still told that the plugin was activated successfully
				// it'd be nice if there was a way to return a message to the user from here
				//if (! $successful ) {
				//	deactivate_plugins('semisecure-login-reimagined/SemisecureLoginReimagined.php', false);
				//}
			}
		}
		
		// intermediate function to generate and store a new RSA keypair
		// the javascript encryption is slightly faster when using 3 as the public exponent
		function storeNewRsaKeypair($numbits = 1024, $publicexponent = 10001) {
			if ($this->generateKeysViaAltMethod()) {
				require_once('SemisecureLoginReimagined_RsaKeys_Alt.php');
				$rsakeys = new SemisecureLoginReimagined_RsaKeys_Alt($numbits);
			}
			else {
				require_once('SemisecureLoginReimagined_RsaKeys.php');
				$rsakeys = new SemisecureLoginReimagined_RsaKeys($numbits, $publicexponent);
			}
			$keys_array = $rsakeys->getKeysArray();

			$successful = (!empty($keys_array) && !empty($keys_array['modulus']) && !empty($keys_array['publicexponent']) && !empty($keys_array['publickey']) && !empty($keys_array['privatekey']) && !empty($keys_array['numbits']));
			if ($successful) {
				// add/update the array to the database (WordPress takes care of serializing the array)
				update_option('semisecurelogin_reimagined_rsa_keys', $keys_array);
			}

			return $successful;
		}

		// Plugin Hook
		// JavaScript to be included in the login page header
		function login_head() {

			// it's important to start the session here for the 'MiniMeta Widget' plugin (possibly other reasons as well)
			@session_start();
			
			// create new login nonce
			$_SESSION['semisecure_login_nonce'] = $this->generateLoginNonce();
			
			$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');

			if (empty($keys_array) OR empty($keys_array['modulus']) OR empty($keys_array['publicexponent']) OR empty($keys_array['privatekey']) OR !extension_loaded('openssl') OR !function_exists('openssl_private_decrypt')) {
				$skipEncrypt = 'true';
			}
			else {
				$skipEncrypt = 'false';
			}

			// Pre-WP2.6 compatibility
			if (!defined('WP_PLUGIN_URL')) {
				define('WP_PLUGIN_URL', get_option('siteurl') . '/wp-content/plugins');
			}

			$plugin_url = WP_PLUGIN_URL . '/' . plugin_basename(dirname(__FILE__));
?>
			<script type="text/javascript" src="<?php echo $plugin_url; ?>/js/jsbn.js"></script>
			<script type="text/javascript" src="<?php echo $plugin_url; ?>/js/prng4.js"></script>
			<script type="text/javascript" src="<?php echo $plugin_url; ?>/js/rng.js"></script>
			<script type="text/javascript" src="<?php echo $plugin_url; ?>/js/rsa.js"></script>
			<script type="text/javascript" src="<?php echo $plugin_url; ?>/js/base64.js"></script>
			<script type="text/javascript">
			function encryptPwd() {
				var formLogin = document.getElementById('loginform');
				var userLog = document.getElementById('user_login');
				var userPwd = document.getElementById('user_pass');
				var password = userPwd.value;

				var skipEncrypt = <?php echo $skipEncrypt; ?>;
				
				var public_n = '<?php echo $keys_array["modulus"]; ?>';
				var public_e = '<?php echo $keys_array["publicexponent"]; ?>';
				
				if (public_n != null && public_e != null && public_n.length > 0 && public_e.length > 0 && !skipEncrypt) {
					semisecureMessage.innerHTML = '<?php _e('Encrypting password &amp; logging in...', 'semisecure-login-reimagined'); ?>';
					var rsa = new RSAKey();
					rsa.setPublic(public_n, public_e);
					var res = rsa.encrypt('<?php echo addslashes($_SESSION['semisecure_login_nonce']); ?>' + password);
					if (res) {
						var userRsaPwd = document.createElement('input');
						userRsaPwd.setAttribute('type', 'hidden');
						userRsaPwd.setAttribute('id', 'pwd_rsa');
						userRsaPwd.setAttribute('name', 'pwd_rsa');
						userRsaPwd.value = hex2b64(res);
						formLogin.appendChild(userRsaPwd);
						
						var temp = '';
						for (var i = 0; i < password.length; i++) {
							temp += '*';
						}
						userPwd.value = temp;
						<?php if ($this->hidePasswordLength) { echo "userPwd.disabled = true;\n"; } else { echo "\n"; } ?>
						return true;
					}
				}
				else {
					return true;
				}
				
				semisecureMessage.innerHTML = '<?php _e('Problem encrypting password!<br/>Please try again or disable JavaScript to login without encryption.', 'semisecure-login-reimagined'); ?>';
				return false;
			}
			// http://onlinetools.org/articles/unobtrusivejavascript/chapter4.html
			function addEvent(obj, evType, fn) {
				if (obj.addEventListener) {
					obj.addEventListener(evType, fn, false);
					return true;
				} else if (obj.attachEvent) {
					var r = obj.attachEvent("on"+evType, fn);
					return r;
				} else {
					return false;
				}
			}
			</script>
<?php
		}

		// Plugin Hook
		// Apply event handlers to the login form (DOM needs to be ready before this happens)
		function login_form() {
			$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
			if (empty($keys_array) OR empty($keys_array["modulus"]) OR empty($keys_array["publicexponent"]) OR empty($keys_array['publicexponent'])) {
				$message = __('Semisecure Login is not enabled!<br />The public key was not found!<br />Your password will not be encrypted!<br />Please login &amp; fix the issue.', 'semisecure-login-reimagined');
			}
			else if(!extension_loaded('openssl') OR !function_exists('openssl_private_decrypt')) {
				$message = __('Semisecure Login is not enabled due to a misconfiguration on the server!<br />Please login &amp; fix the issue.', 'semisecure-login-reimagined');
			}
			else {
				$message = __('Semisecure Login is enabled.', 'semisecure-login-reimagined');
			}
?>
			<p id="semisecure-message">
				<?php _e('Semisecure Login is not enabled!<br />Please enable JavaScript and use a modern browser to ensure that your password is encrypted.', 'semisecure-login-reimagined'); ?>
			</p>
			<script type="text/javascript">
				var formLogin = document.getElementById('loginform');
				formLogin.setAttribute('onsubmit', 'javascript:return encryptPwd();');
				addEvent(window, 'click', rng_seed_time);
				addEvent(window, 'keypress', rng_seed_time);
				
				var semisecureMessage = document.getElementById('semisecure-message');
				semisecureMessage.setAttribute('class', '');
				semisecureMessage.innerHTML = '<?php echo $message; ?>';
			</script>
<?php
		}

		// Plugin Hook
		// decrypt the encrypted password and pass back to wp_authenticate
		function semisecure_authenticate(&$username, &$password) {

			// calling session_start here solves a problem where sometimes you would have to reload the page (wp-login.php) to properly start the session
			// previously this issue was fixed by starting the session using an 'init' hook, but that doesn't appear to be necessary (on the other hand... sometimes in WP2.6.2 this isn't enough)
			@session_start();

			// if set, this decrypts our encrypted password
			// otherwise degrades in the absence of javascript
			if (!empty($_POST['pwd_rsa'])) {
				$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
				$privkey = $keys_array['privatekey'];
				if (!empty($privkey)) {
					$success = @openssl_private_decrypt(base64_decode(stripslashes($_POST['pwd_rsa'])), $decrypted_pass, $privkey, OPENSSL_PKCS1_PADDING);
					if ($success) {
						$login_nonce_length = strlen($_SESSION['semisecure_login_nonce']);
						if ($login_nonce_length > 0 && substr($decrypted_pass, 0, $login_nonce_length) == $_SESSION['semisecure_login_nonce']) {
							$password = substr($decrypted_pass, $login_nonce_length);
						}
					}
				}
			}
			
			// expire the login nonce
			$_SESSION['semisecure_login_nonce'] = $this->generateLoginNonce();

		}
		
		// add the settings page
		function add_pages() {
			add_options_page(__('Semisecure Login Reimagined', 'semisecure-login-reimagined'), __('Semisecure Login', 'semisecure-login-reimagined'), 'manage_options', 'semisecureloginreimagined', array(&$this, 'options_page'));
		}
		
		// display the settings page
		function options_page() {
			// generate and save a new keypair if postback
			if (isset($_POST['rsa_numbits']) && !empty($_POST['rsa_numbits']) && is_numeric($_POST['rsa_numbits'])) {
				if (current_user_can('manage_options')) {
					check_admin_referer('semisecure-login-reimagined_update');
					$rsa_numbits = $_POST['rsa_numbits'];
					$rsa_publicexponent = is_numeric($_POST['rsa_publicexponent']) ? $_POST['rsa_publicexponent'] : 10001;
					$successful = $this->storeNewRsaKeypair($rsa_numbits, $rsa_publicexponent);
					if ($successful)
						print '<div id="message" class="updated fade"><p><strong>' . __('The new keypair was successfully generated and saved.', 'semisecure-login-reimagined') . '</strong></p></div>';
					else
						print '<div id="message" class="error"><p><strong>' . __('Keypair could not be created!', 'semisecure-login-reimagined') . '</strong></p></div>';
				}
				else {
					print '<div id="message" class="error"><p><strong>' . __('Ooops... looks like you don\'t have the correct privileges to make this change.', 'semisecure-login-reimagined') . '</strong></p></div>';
				}
			}
			// uninstall
			else if (isset($_POST['uninstall_semisecure_login_reimagined']) && !empty($_POST['uninstall_semisecure_login_reimagined']) && $_POST['uninstall_semisecure_login_reimagined'] == 'yes') {
				if (current_user_can('manage_options')) {
					check_admin_referer('semisecure-login-reimagined_uninstall');
					delete_option('semisecurelogin_reimagined_rsa_keys');
					$deactivate_url = 'plugins.php?action=deactivate&amp;plugin=semisecure-login-reimagined%2FSemisecureLoginReimagined.php';
					if(function_exists('wp_nonce_url')) { 
						$deactivate_url = wp_nonce_url($deactivate_url, 'deactivate-plugin_semisecure-login-reimagined/SemisecureLoginReimagined.php');
					}
					print '<div id="message" class="updated fade"><p><strong>' . sprintf(__('<a href="%s">Click here</a> to finish the uninstallation and this plugin will be deactivated automatically.', 'semisecure-login-reimagined'), $deactivate_url) . '</strong></p></div>';
				}
				else {
					print '<div id="message" class="error"><p><strong>' . __('Ooops... looks like you don\'t have the correct privileges to make this change.', 'semisecure-login-reimagined') . '</strong></p></div>';
				}
			}
			
			$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
?>
			<div class="wrap">
			<h2><?php _e('Semisecure Login Reimagined', 'semisecure-login-reimagined'); ?></h2>
<?php
			// check if PHP has loaded openssl
			if (!extension_loaded('openssl')) {
?>
				<p style="color:red;">
					<strong><?php _e('WARNING:', 'semisecure-login-reimagined'); ?></strong><br />
					<?php _e('OpenSSL doesn\'t appear to be installed. This plugin relies on OpenSSL and won\'t work until it\'s been installed.', 'semisecure-login-reimagined'); ?>
				</p>
<?php
			// check if our decrypt function is available
			} else if(!function_exists('openssl_private_decrypt')) {
?>
				<p style="color:red;">
					<strong><?php _e('WARNING:', 'semisecure-login-reimagined'); ?></strong><br />
					<?php _e('The <em>openssl_private_decrypt</em> function appears to be disabled. This plugin won\'t work until it\'s been enabled.<br />Check the <em>disable_functions</em> setting in your php.ini file.', 'semisecure-login-reimagined'); ?>
				</p>
<?php
			}
?>
				<p>
					<?php _e('This plugin uses RSA public-key encryption to encrypt your password on the client-side with the public key, and then decrypt it on the server-side with the private key. A nonce is used to help prevent replay attacks. This provides a &quot;semisecure&quot; login environment. For full security you should use an SSL certificate.', 'semisecure-login-reimagined'); ?>
				</p>
				<table border="0" style="float:left;margin-left:10px;">
					<tr>
						<th colspan="2"><?php _e('512 bits', 'semisecure-login-reimagined'); ?></th>
					</tr>
					<tr>
						<td>&nbsp;</td>
						<td><?php $chars = 64; printf(__('%d characters', 'semisecure-login-reimagined'), $chars); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php _e('11 characters PKCS#1 padding', 'semisecure-login-reimagined'); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php printf(__('%d character nonce', 'semisecure-login-reimagined'), $this->nonceLength); ?></td>
					</tr>
					<tr>
						<td> = </td>
						<td><?php printf(__('%d max password characters', 'semisecure-login-reimagined'), $chars - 11 - $this->nonceLength); ?></td>
					</tr>
				</table>
				<table border="0" style="float:left;margin-left:10px;">
					<tr>
						<th colspan="2"><?php _e('1024 bits', 'semisecure-login-reimagined'); ?></th>
					</tr>
					<tr>
						<td>&nbsp;</td>
						<td><?php $chars = 128; printf(__('%d characters', 'semisecure-login-reimagined'), $chars); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php _e('11 characters PKCS#1 padding', 'semisecure-login-reimagined'); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php printf(__('%d character nonce', 'semisecure-login-reimagined'), $this->nonceLength); ?></td>
					</tr>
					<tr>
						<td> = </td>
						<td><?php printf(__('%d max password characters', 'semisecure-login-reimagined'), $chars - 11 - $this->nonceLength); ?></td>
					</tr>
				</table>
				<table border="0" style="float:left;margin-left:10px;">
					<tr>
						<th colspan="2"><?php _e('2048 bits', 'semisecure-login-reimagined'); ?></th>
					</tr>
					<tr>
						<td>&nbsp;</td>
						<td><?php $chars = 256; printf(__('%d characters', 'semisecure-login-reimagined'), $chars); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php _e('11 characters PKCS#1 padding', 'semisecure-login-reimagined'); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php printf(__('%d character nonce', 'semisecure-login-reimagined'), $this->nonceLength); ?></td>
					</tr>
					<tr>
						<td> = </td>
						<td><?php printf(__('%d max password characters', 'semisecure-login-reimagined'), $chars - 11 - $this->nonceLength); ?></td>
					</tr>
				</table>
				<table border="0" style="float:left;margin-left:10px;">
					<tr>
						<th colspan="2"><?php _e('3072 bits', 'semisecure-login-reimagined'); ?></th>
					</tr>
					<tr>
						<td>&nbsp;</td>
						<td><?php $chars = 384; printf(__('%d characters', 'semisecure-login-reimagined'), $chars); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php _e('11 characters PKCS#1 padding', 'semisecure-login-reimagined'); ?></td>
					</tr>
					<tr>
						<td> - </td>
						<td><?php printf(__('%d character nonce', 'semisecure-login-reimagined'), $this->nonceLength); ?></td>
					</tr>
					<tr>
						<td> = </td>
						<td><?php printf(__('%d max password characters', 'semisecure-login-reimagined'), $chars - 11 - $this->nonceLength); ?></td>
					</tr>
				</table>
				<div style="clear:both;"></div>
				<p>
					<?php _e('A lower number of bits requires less processing power, but also limits the amount of data it can encode. More bits corresponds to both better encryption and higher processing load.', 'semisecure-login-reimagined'); ?>
				</p>
				<p>
					<?php _e('<em>&quot;RSA claims that 1024-bit keys are likely to become crackable some time between 2006 and 2010 and that 2048-bit keys are sufficient until 2030. An RSA key length of 3072 bits should be used if security is required beyond 2030.&quot;</em> -<a href="http://en.wikipedia.org/wiki/Key_size#Asymmetric_algorithm_key_lengths" target="_blank">Wikipedia</a>', 'semisecure-login-reimagined'); ?>
				</p>
				<h3><?php _e('Current Keypair', 'semisecure-login-reimagined'); ?></h3>
				<p>
<?php
			// check if a keypair already exists
			if (!empty($keys_array) && !empty($keys_array['modulus']) && !empty($keys_array['publicexponent']) && !empty($keys_array['publickey']) && !empty($keys_array['privatekey']) && !empty($keys_array['numbits'])) {
?>
				<?php printf(__('The current key is <strong>%d</strong> bits in length, and the public-exponent is <strong>%s</strong>', 'semisecure-login-reimagined'), $keys_array['numbits'], '0x' . $keys_array['publicexponent']); ?>
				<table border="0">
					<tr valign="top">
						<td><?php _e('Modulus:', 'semisecure-login-reimagined'); ?></td>
						<td><pre style="padding-top:0;margin-top:0;"><?php echo wordwrap($keys_array['modulus'], 64, "\n", true) ?></pre></td>
					</tr>
				</table>
<?php
			} else {
?>
				<span style="color:red;"><strong><?php _e('WARNING:', 'semisecure-login-reimagined'); ?></strong><br /><?php _e('No current keypair found! Please remedy the situation by generating a new keypair below.', 'semisecure-login-reimagined'); ?></span>
<?php
			}
?>
			</p>
			<h3><?php _e('Generate New Keypair', 'semisecure-login-reimagined'); ?></h3>
			<p>
				<?php _e('Use the following options to generate &amp; store a new RSA keypair', 'semisecure-login-reimagined'); ?>
			</p>
			<form method="post" action="">
				<table class="form-table">
					<tr valign="top">
						<th scope="row"><?php _e('Number of bits', 'semisecure-login-reimagined'); ?></th>
						<td>
							<select name="rsa_numbits">
								<option value="512"><?php _e('512 bits', 'semisecure-login-reimagined'); ?></option>
								<option value="1024" selected="selected"><?php _e('1024 bits', 'semisecure-login-reimagined'); ?></option>
								<option value="2048"><?php _e('2048 bits', 'semisecure-login-reimagined'); ?></option>
								<option value="3072"><?php _e('3072 bits', 'semisecure-login-reimagined'); ?></option>
							</select>
							<br/><?php _e('1024 bits is currently recommended based on the tradeoff between performance and security.', 'semisecure-login-reimagined'); ?>
						</td>
					</tr>
<?php
					if ($this->generateKeysViaAltMethod()) {
						$disabled_text = ' disabled="disabled"';
						$after_text = __('Attempting to use the alternative keypair generation method. This alternative method does not let you choose the public-exponent.  ', 'semisecure-login-reimagined');
						if ($this->forceAltKeypairGenerationMethod)
							$after_text .= __('This method is being attempted because you enabled an <abbr title="Advanced options are set directly in the AdvancedOptions file">advanced option</abbr>.', 'semisecure-login-reimagined');
						else
							$after_text .= __('This method is being attempted because the PHP execution functions have been disabled.', 'semisecure-login-reimagined');
						$after_text .= sprintf(__('  To successfully generate a keypair using this method, PHP 5.2.0 or greater must be used.  You appear to be using PHP %s.', 'semisecure-login-reimagined'), PHP_VERSION);
					}
					else {
						$disabled_text = '';
						$after_text = __('OpenSSL defaults to F4; 0x3 has slightly faster encryption times', 'semisecure-login-reimagined');
					}
?>
					<tr valign="top">
						<th scope="row"><?php _e('Public-exponent', 'semisecure-login-reimagined'); ?></th>
						<td>
							<select name="rsa_publicexponent"<?php echo $disabled_text; ?>>
								<option value="3"><?php _e('3 (0x3)', 'semisecure-login-reimagined'); ?></option>
								<option value="10001" selected="selected"><?php _e('F4 (0x10001)', 'semisecure-login-reimagined'); ?></option>
							</select>
							<br/><?php echo $after_text; ?>
						</td>
					</tr>
				</table>
				<p class="submit">
					<?php wp_nonce_field('semisecure-login-reimagined_update'); ?>
					<input type="submit" name="Submit" value="<?php _e('Update Options &raquo;', 'semisecure-login-reimagined'); ?>" />
				</p>
			</form>
			</div>
			<p>&nbsp;</p>
			<div class="wrap">
			<h2><?php _e('Uninstall', 'semisecure-login-reimagined'); ?></h2>
			<p>
				<?php _e('Deactivating this plugin does not remove any data that may have been created, such as the RSA keypair. To completely remove this plugin, you can uninstall it here.', 'semisecure-login-reimagined'); ?>
			</p>
			<p style="color:red;">
				<strong><?php _e('WARNING:', 'semisecure-login-reimagined'); ?></strong><br /><?php _e('Once uninstalled, this cannot be undone. Make sure this is what you want to do. Optionally, you should backup your WordPress database first.', 'semisecure-login-reimagined'); ?>
			</p>
			<p>
				<?php _e('As of WordPress 2.7, if you delete this plugin through the admin interface, it will automatically clean up the following option(s) from the database.', 'semisecure-login-reimagined'); ?>
			</p>
			<p>
				<?php _e('Do you want to remove the following option(s) from the database?', 'semisecure-login-reimagined'); ?>
			</p>
			<table class="widefat">
			<thead>
				<tr>
					<th><?php _e('WordPress Options', 'semisecure-login-reimagined'); ?></th>
				</tr>
			</thead>
				<tr>
					<td valign="top">
						<ol>
						<li>semisecurelogin_reimagined_rsa_keys</li>
						</ol>
					</td>

				</tr>
			</table>
			<form method="post" action="">
			<p style="text-align: center;">
				<label><input type="checkbox" name="uninstall_semisecure_login_reimagined" value="yes" /> <?php _e('Yes', 'semisecure-login-reimagined'); ?> </label><br /><br />
				<?php wp_nonce_field('semisecure-login-reimagined_uninstall'); ?>
				<input class="button" type="submit" name="Submit" value="<?php _e('UNINSTALL &raquo;', 'semisecure-login-reimagined'); ?>" />
			</p>
			</form>
			</div>
<?php
		}

		// generates a random login nonce
		function generateLoginNonce() {
			$length = $this->nonceLength;
			$array = str_shuffle($this->nonceCharacters);
			$max = strlen($array) - 1;
			$nonce = '';
			for ($i = 0; $i < $length; $i++) {
				if (function_exists('wp_rand')) { // wp_rand() was added in WP2.6.2
					$nonce .= $array{wp_rand(0, $max)};
				}
				else if ($fh = @fopen('/dev/urandom', 'r')) { // this should work on Linux servers, won't work on Windows
					$urandom = @fread($fh, 4); // 256^4 - 1 = 4,294,967,295
					@fclose($fh);
					$nonce .= $array{fmod(hexdec(bin2hex($urandom)), ($max + 1))};
				}
				else { // fallback to something we know will work, but might possibly be the least random of the 3
					$nonce .= $array{mt_rand(0, $max)}; // mt_rand() might be more random than rand() on Windows, on Linux they should be about the same
				}
			}
			return $nonce;
		}

		// lets us know which method of RSA key generation we should use
		function generateKeysViaAltMethod() {
			if ($this->forceAltKeypairGenerationMethod || ini_get('safe_mode') || !function_exists('proc_open') || !function_exists('proc_close') || !function_exists('shell_exec')) {
				return true;
			}
			else {
				return false;
			}
		}
		
	}
}

$semisecureLoginReimagined = new SemisecureLoginReimagined();
?>