<?php
// this class includes advanced editable options that aren't available in the admin interface
// don't edit these options unless you know what you're doing!
// keep in mind that when upgrading this plugin, this file will be overwritten, and you will need to "re-make" your changes to the following advanced options
// future versions of this file might contain additional options, and choosing NOT to update this file may break the plugin
if (!class_exists('SemisecureLoginReimagined_AdvancedOptions')) {
	class SemisecureLoginReimagined_AdvancedOptions {

		/** ******************************* START EDITING!!! ******************************** **/

		// Force the alternate (PHP only) keypair generation method
		// This alternate method requires at least PHP 5.2.0
		// Normally the alt method is only used if the PHP execution functions have been disabled
		// Allowed values: TRUE or FALSE (the default value is FALSE)
		var $forceAltKeypairGenerationMethod = FALSE;

		// Should we hide the password length completely when logging in (which is potentially more secure, if someone knows your password length it makes brute force login attacks easier)
		//   or should we submit the password as a series of asterisks (*****) with the same length as the password
		// Hiding the password length disables the password field right before the form is submitted (most browsers will show the password field as "greyed out" when this happens)
		// If this is true then you might get a "blank password" message when normally you would get an "incorrect password" message
		// Allowed values: TRUE or FALSE (the default value is TRUE)
		var $hidePasswordLength = TRUE;
		// Note: this NEEDS to be set to FALSE if you're using WP2.5 (it looks like there was a bug introduced in this version that was fixed in WP2.5.1)

		// This should be used if, for some reason, the session isn't properly initializing (because the headers have already been sent, for example) and it's rejecting your password
		// If you choose to force session start before sending headers then you can choose to do this only for the login page or for all pages
		// This most likely won't need to be used (at least under a default setup), but it's here as a failsafe
		// The login page is wp-login.php
		var $forceSessionStartBeforeSendingHeaders = TRUE; // TRUE or FALSE (defaults to TRUE)
		var $forceSessionStartOnlyForLoginPage = TRUE;      // TRUE or FALSE (defaults to TRUE)

		// If you're using the default keypair generation method, and for some reason the path to openssl isn't included in your server's system path
		//   (which means that you can't call out directly to openssl)
		//   then you can include the full path to openssl here
		// For example: '/usr/bin/openssl' or 'C:\\OpenSSL\\bin\\openssl'
		// This is a string (the default value is 'openssl')
		var $opensslLocation = 'openssl';

		// Here, you can update the nonce specifics
		// The default nonce length is 10
		//   and these are the default characters that are randomly chosen from to create the nonce:
		//   '!@#$%^&*()-_[]{}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
		// $nonceLength is an integer
		// $nonceCharacters is a string of unique characters
		// Don't use certain special characters, such as newline/linefeed (\n) or carriage return (\r)
		var $nonceLength = 10;
		var $nonceCharacters = '!@#$%^&*()-_[]{}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';

		// This option is helpful if you're using a 2nd (or 3rd, etc.) plugin that also outputs "something" below the login form
		// For example, if you're using the WP-OpenID plugin, then the login form will display both the "Semisecure" message and the OpenID textbox below the login form
		//   This option will explicitly let you choose if the "Semisecure" message is displayed before or after the OpenID textbox
		// A smaller number will cause the "Semisecure" message to be displayed earlier (before "something else"), and a larger number will cause the message to be displayed later (after "something else")
		//   This number is relative to that "something else"
		// This is an integer, and the default value is 10 (which corresponds to WordPress' default value for plugin hooks)
		var $loginFormSemisecureMessagePosition = 10;

		/** ******************************** STOP EDITING!!! ******************************** **/

	}
}
?>